
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of libdap, A C++ implementation of the OPeNDAP Data
// Access Protocol.

// Copyright (c) 2002,2003,2005 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//         Reza Nekovei <reza@intcomm.net>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.

// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// netCDF sub-class implementation for NCByte,...NCGrid.
// The files are patterned after the subcalssing examples 
// Test<type>.c,h files.
//
// ReZa 1/12/95

#ifndef _ncarray_h
#define _ncarray_h 1


#include <sstream>

#include "Array.h"
#include "NCAccess.h"
#include "DimProjInfo.h"

using namespace libdap ;

class NCArray: public Array, public NCAccess {
    
    // Assume there's a one-to-one mapping between Array dimensions and 
    // elements in this vector. If there are fewer elements in this vector
    // than dimensions in the Array, assume the 'later' dimensions are not
    // constrained at all, even though that is an invalid CE and should 
    // cause the server to return an Error.
    vector<dim_proj_info> cl_proj;
    
    void m_duplicate(const NCArray &nca);
    
public:
    NCArray(const string &n, BaseType *v);
    NCArray(const NCArray &nc_array);
    NCArray &operator=(const NCArray &rhs);
    virtual ~NCArray();

    virtual BaseType *ptr_duplicate();

    virtual string build_constraint(int outtype, const size_t *start,
            const size_t *edges, const ptrdiff_t *stride) throw(Error);

    virtual void store_projection(const string &proj);            
    virtual bool is_convertable(int outtype);

    virtual nc_type get_nc_type() throw(InternalErr);
    
    virtual void extract_values(void *values, int elements, int outtype,
                                const ClientParams &/* cp */) throw(Error);
    // virtual BaseType *get_source();
    virtual void set_source(BaseType *s) throw(InternalErr);
    
    virtual VarList flatten(const ClientParams &cp, const string &parent_name);    
    /// Does this variable contain a Sequence?
    virtual BaseType *find_child_sequence();
};

#endif

