
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of libdap, A C++ implementation of the OPeNDAP Data
// Access Protocol.

// Copyright (c) 2002,2003,2005 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//         Reza Nekovei <reza@intcomm.net>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.

// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// netCDF sub-class implementation for NCByte,...NCGrid.
// The files are patterned after the subcalssing examples 
// Test<type>.c,h files.
//
// ReZa 1/12/95

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: NCFloat64.cc 19625 2008-09-17 23:00:57Z jimg $"};


#include "InternalErr.h"

#include "NCSequence.h"
#include "NCFloat64.h"
#include "nc_util.h"
#include "Dnetcdf.h"

void 
NCFloat64::m_duplicate(const NCFloat64 &bt)
{
    dynamic_cast<NCAccess&>(*this).clone(dynamic_cast<const NCAccess&>(bt));
}

NCFloat64::NCFloat64(const string &n) : Float64(n)
{
}

NCFloat64::NCFloat64(const NCFloat64 &rhs) : Float64(rhs)
{
    m_duplicate(rhs);
}

NCFloat64::~NCFloat64()
{
}

NCFloat64 &
NCFloat64::operator=(const NCFloat64 &rhs)
{
    if (this == &rhs)
        return *this;

    dynamic_cast<NCFloat64&>(*this) = rhs;

    m_duplicate(rhs);

    return *this;
}

BaseType *
NCFloat64::ptr_duplicate()
{
    return new NCFloat64(*this); // Copy ctor calls duplicate to do the work
}
 
nc_type
NCFloat64::get_nc_type() throw(InternalErr)
{
    return NC_DOUBLE;
}

