
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of libdap, A C++ implementation of the OPeNDAP Data
// Access Protocol.

// Copyright (c) 2002,2003,2005 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//         Reza Nekovei <reza@intcomm.net>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.

// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// netCDF sub-class implementation for NCByte,...NCGrid.
// The files are patterned after the subcalssing examples 
// Test<type>.c,h files.
//
// ReZa 1/12/95

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: NCStr.cc 19625 2008-09-17 23:00:57Z jimg $"};

// #define DODS_DEBUG 1

#include "InternalErr.h"
#include "NCStr.h"
#include "NCSequence.h"

#include "debug.h"

#include "Dnetcdf.h"

void 
NCStr::m_duplicate(const NCStr &bt)
{
    dynamic_cast<NCAccess&>(*this).clone(dynamic_cast<const NCAccess&>(bt));
}


NCStr::NCStr(const string &n) : Str(n)
{
}

NCStr::NCStr(const NCStr &rhs) : Str(rhs)
{
    m_duplicate(rhs);
}

NCStr::~NCStr()
{
}

NCStr &
NCStr::operator=(const NCStr &rhs)
{
    if (this == &rhs)
        return *this;

    dynamic_cast<NCStr&>(*this) = rhs;

    m_duplicate(rhs);

    return *this;
}

BaseType *
NCStr::ptr_duplicate()
{
    return new NCStr(*this);
}

void
NCStr::extract_values(void *values, int elements, int /*outtype*/,
                      const ClientParams &cp) throw(Error)
{
    // If this variable is held by a sequence, we need to treat it specially.
    // Read values from the Sequence and then use this instance to process
    // the values. 
    NCSequence *ncq = dynamic_cast<NCSequence*>(find_ancestral_sequence(this));
    int nels = (ncq) ? ncq->number_of_rows() : 1;
    unsigned int dimsize=get_strdim();
    
    DBG(cerr << "nels: " << nels << endl);
    
    char * tbfr = (char *)values;
    int i = 0;
    while (i < nels) {
        string *sp = 0;
        string **spp = &sp;
        // If the parent of this scalar is a Sequence, use that Sequence
        // to read values.
        if (ncq) {
            // replace name() with index. 9/10/2004
            NCStr *s = dynamic_cast<NCStr*>(ncq->var_value(i, name()));
            if (!s)
                throw InternalErr(__FILE__, __LINE__, "Bad csat to NCStr.");
            s->buf2val((void **)spp);
        }
        else {     
            buf2val((void **)spp);
        }
        
        if (cp.get_string_as_char_array() && get_translated()) {
            unsigned int c = 0;
            while (c < dimsize
                   && (c < sp->length()
                       || (sp->length() == 0 && c == 0))) {
                *tbfr++ = *(sp->c_str() + c++);
            }
            while (c++ < dimsize ) {
                *tbfr++ = 0;
            }
        }
        else {
            for (unsigned int c = 0;
                 c < sp->length() || (sp->length() == 0 && c == 0);
                 c++) {
                 // I think it may be a bug to transfer more than one char since
                 // the client program has allocated a nels buffer for the data.
                 // jhrg 2/24/05
                 *tbfr++ = *(sp->c_str() + c);
            }
        }

        ++i;
        DBG(cerr << "Value: " << *sp << endl);
        // Now get rid of the C++ string object.
        delete sp;
    }
    while (i++ < elements) {
        if (cp.get_string_as_char_array() && get_translated()) {
            unsigned int c = 0;
            while (c++ < dimsize) {
                *tbfr++ = 0;
            }
        }
        else {
            *tbfr++ = 0;
        }
    }
}

nc_type
NCStr::get_nc_type() throw(InternalErr)
{
    return NC_CHAR;
}

